"""Fill in a module description here"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/client.ipynb.

# %% auto 0
__all__ = ['ExecutionError']

# %% ../nbs/client.ipynb 2
class ExecutionError(Exception):
    def __init__(self, message):
        super().__init__(message)



"""Fill in a module description here"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/LocalAccount/00_DJ_WorkspaceAccount.ipynb.

# %% auto 0
__all__ = ['DJA_UpdateCredsError', 'DJ_WorkspaceAccount']

# %% ../nbs/LocalAccount/00_DJ_WorkspaceAccount.ipynb 3
class DJA_UpdateCredsError(dlcr.ClassError):
    def __init__(self, cred_type, local_account_name, additional_message=None):
        message = f"unable to update {cred_type} for - {local_account_name}"

        if additional_message:
            message = f"{message}\n\n{additional_message}"

        super().__init__(message)

# %% ../nbs/LocalAccount/00_DJ_WorkspaceAccount.ipynb 4
import domojupyter.classes.DomoJupyter as dmdj

@dataclass
class DJ_WorkspaceAccount(dmdj.DomoJupyterAccount):
    target_instance: str

    dj_workspace: Any = field(repr=False)  # DJ_Workspace

    local_account_name: str
    local_domo_account: dmacc.DomoAccount = field(repr=False, default=None)
    local_creds: dict = field(repr=False, default=None)

    is_valid_full_auth: bool = None
    is_valid_token_auth: bool = None

    is_domo_account_exists: bool = False

    _target_token_auth: dmda.DomoAuth = None
    _target_full_auth: dmda.DomoAuth = None

    target_auth: dmda.DomoAuth = field(repr=False, default=None)
    backup_auth: dmda.DomoAuth = field(repr=False, default=None)

    username: str = None
    password: str = field(repr=False, default=None)
    access_token: str = field(repr=False, default=None)

    account_config: dmacc.AccountConfig = None

    


    @classmethod
    def from_workspace_by_account_name(
        cls,
        domo_instance: str,
        local_account_name: str,
        dj_workspace: Any,
    ):
        return cls(
            target_instance=domo_instance,
            local_account_name=local_account_name,
            dj_workspace=dj_workspace,
        )

# %% ../nbs/LocalAccount/00_DJ_WorkspaceAccount.ipynb 6
@patch_to(DJ_WorkspaceAccount)
async def _search_domo_groups_by_name(
    self, auth: dmda.DomoAuth, group_names: List[str]
):
    await dmdg.DomoGroups.toggle_system_group_visibility(
        is_hide_system_groups=False, auth=auth
    )

    await asyncio.sleep(1)

    domo_groups = await dmce.gather_with_concurrency(
        *[
            dmdg.DomoGroup.search_by_name(group_name=group_name, auth=auth)
            for group_name in group_names
        ],
        n=10,
    )
    await dmdg.DomoGroups.toggle_system_group_visibility(
        is_hide_system_groups=True, auth=auth
    )
    return domo_groups


@patch_to(DJ_WorkspaceAccount)
async def _upsert_group(
    self,
    auth: dmda.DomoAuth,
    group_name: str,
    group_owner_names: list[str] = [
        "Role: sie_instance_admin",
        "Role: sie_dde_instance_admin",
    ],
    debug_api: bool = False,
):

    domo_group = await dmdg.DomoGroup.upsert(
        group_name=group_name,
        group_type=dmdg.GroupType_Enum["CLOSED"].value,
        description=f"updated via {dt.date.today()}",
        auth=auth,
        debug_api=debug_api,
    )

    domo_group_owners = await self._search_domo_groups_by_name(
        auth=auth, group_names=group_owner_names
    )

    await domo_group.Membership.add_owners(add_owner_ls=domo_group_owners)

    return domo_group


@patch_to(DJ_WorkspaceAccount)
def _generate_account_config(self):
    account_config = dmacc.AccountConfig.domo_access_token.value(
        username=self.username,
        password=self.password,
        domo_access_token=self.domo_access_token,
    )

    return account_config


@patch_to(DJ_WorkspaceAccount)
async def _upsert_account_and_share_with_default_group(
    self,
    account_config: dmacc.AccountConfig,
    account_name: str,
    auth: dmda.DomoAuth,
    debug_api: bool = False,
    default_share_group: dmdg.DomoGroup = None,
):

    domo_account = await dmacc.DomoAccounts.upsert_account(
        account_config=account_config,
        account_name=account_name,
        auth=auth,
        debug_api=debug_api,
    )

    if default_share_group:
        share_res = await domo_account._share_v2(group_id=default_share_group.id)

    return domo_account


@patch_to(DJ_WorkspaceAccount)
async def _upsert_account(
    self,
    auth: dmda.DomoAuth,
    account_name: str,
    default_share_group_name: str = None,
    default_share_group_owner_names=[
        "Role: sie_instance_admin",
        "Role: sie_dde_instance_admin",
    ],
    debug_api: bool = False,
    is_test: bool = False,
):
    account_config = self._generate_account_config()

    group_name = default_share_group_name or self.dj_workspace.default_share_group_name

    default_share_group = None

    if group_name:
        default_share_group = await self._upsert_group(
            auth=auth,
            group_name=group_name,
            group_owner_names=default_share_group_owner_names
            or self.default_share_group_name,
            debug_api=debug_api,
        )

    if is_test:
        return account_config, default_share_group

    domo_account = await self._upsert_account_and_share_with_default_group(
        auth=auth,
        account_config=account_config,
        account_name=account_name,
        default_share_group=default_share_group,
        debug_api=debug_api,
    )

    return domo_account


@patch_to(DJ_WorkspaceAccount)
async def upsert_local_account(self, is_test=True, debug_api: bool = False):
    try:
        return await self._upsert_account(
            auth=self.dj_workspace.local_auth,
            account_name=self.local_account_name,
            is_test=is_test,
            debug_api=debug_api,
        )
    except dmde.DomoError as e:
        print(e)
        return None


@patch_to(DJ_WorkspaceAccount)
async def upsert_target_account(
    self,
    target_account_name: str,
    backup_auth: dmda.DomoAuth = None,
    is_test=True,
    debug_api: bool = False,
):
    try:
        target_auth = await self.generate_target_auth(backup_auth=backup_auth)
    except DJA_ValidAuth as e:
        raise DJA_UpdateCredsError(
            cred_type="target_account",
            local_account_name=target_account_name,
            additional_message=str(e),
        ) from e

        return await self._upsert_account(
            auth=target_auth,
            account_name=target_account_name,
            is_test=is_test,
            debug_api=debug_api,
        )

    except dmde.DomoError as e:
        print(e)
        return None


@patch_to(DJ_WorkspaceAccount)
async def upsert_remote_account(
    self,
    remote_account_name: str,
    auth: dmda.DomoAuth,
    is_test=True,
    debug_api: bool = False,
):
    try:
        return await self._upsert_account(
            auth=auth,
            account_name=remote_account_name,
            is_test=is_test,
            debug_api=debug_api,
        )
    except dmde.DomoError as e:
        print(e)
        return None


@patch_to(DJ_WorkspaceAccount)
async def upsert_accounts(
    self,
    target_account_name: str = None,
    target_auth_backup: dmda.DomoAuth = None,
    remote_account_name: str = None,
    remote_auth_ls: List[dmda.DomoAuth] = None,
    debug_api: bool = False,
):
    """attempts to upsert account in target instance AND deploy account to remote instances"""
    target_acc = await self.upsert_target_account(
        target_account_name=target_account_name or self.local_account_name,
        backup_auth=target_auth_backup,
        is_test=False,
        debug_api=debug_api,
    )

    local_acc = await self.upsert_local_account(is_test=False, debug_api=debug_api)

    remote_accs = None
    if remote_auth_ls:
        remote_accs = await dmce.gather_with_concurrency(
            *[
                self.upsert_remote_account(
                    remote_account_name=remote_account_name or self.local_account_name,
                    auth=r_auth,
                )
                for r_auth in remote_auth_ls
            ],
            n=10,
        )

    return {"target": target_acc, "local": local_acc, "remotes": remote_accs}

# %% ../nbs/LocalAccount/00_DJ_WorkspaceAccount.ipynb 9
class AccessToken_NotExpired(dlcr.ClassError):
    def __init__(self, token_name):
        super().__init__(f"token {token_name} is not expired")


@patch_to(DJ_WorkspaceAccount)
async def regenerate_access_token(
    self,
    domo_user: dmdu.DomoUser,
    auth: dmda.DomoAuth,
    token_name: str,
    duration_in_days: int = 90,
    access_token_days_to_expiry_threshold: int = 7,
    is_force_regeneration: bool = False,
    raise_exception: bool = True,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
):

    domo_config = dmic.DomoInstanceConfig(auth=auth)

    if not is_force_regeneration:
        domo_access_tokens = await domo_config.get_access_tokens(
            debug_api=debug_api, session=session
        )

        domo_access_token = next(
            (
                token
                for token in domo_access_tokens
                if token.owner == domo_user and token.name == token_name
            ),
            None,
        )

        if (
            domo_access_token
            and access_token_days_to_expiry_threshold
            <= (domo_access_token.expiration_date - dt.datetime.now()).days
        ):
            if raise_exception:
                raise AccessToken_NotExpired(token_name)

            return domo_access_token

    domo_access_token = await domo_config.regenerate_access_token(
        domo_user=domo_user,
        token_name=token_name,
        session=session,
        duration_in_days=duration_in_days,
        debug_api=debug_api,
    )
    self.domo_access_token = domo_access_token.token

    return domo_access_token


@patch_to(DJ_WorkspaceAccount)
async def regenerate_target_access_token(
    self,
    token_name: str = None,
    duration_in_days: int = 90,
    access_token_days_to_expiry_threshold: int = 7,
    is_force_regeneration: bool = False,
    debug_api: bool = False,
    trigger_upsert_accounts: bool = True,
    target_account_name: str = None,
    target_auth_backup: dmda.DomoAuth = None,
    remote_account_name: str = None,
    remote_auth_ls: List[dmda.DomoAuth] = None,
):

    if not self.is_valid_token_auth:
        is_force_regeneration = True

    target_auth = await self.generate_target_auth(backup_auth=target_auth_backup)

    domo_user = await dmdu.DomoUsers.by_email(
        email_ls=[self.username], auth=target_auth
    )

    domo_access_token = await self.regenerate_access_token(
        domo_user=domo_user,
        auth=target_auth,
        token_name=token_name or self.local_account_name,
        duration_in_days=duration_in_days,
        access_token_days_to_expiry_threshold=access_token_days_to_expiry_threshold,
        is_force_regeneration=is_force_regeneration,
        raise_exception=False,
        debug_api=debug_api,
    )

    if trigger_upsert_accounts:
        return await self.upsert_accounts(
            target_account_name=target_account_name or self.local_account_name,
            target_auth_backup=target_auth_backup,
            remote_account_name=remote_account_name or self.local_account_name,
            remote_auth_ls=remote_auth_ls,
            debug_api=debug_api,
        )

    return domo_access_token

# %% ../nbs/LocalAccount/00_DJ_WorkspaceAccount.ipynb 10
class AccessToken_NotExpired(dlcr.ClassError):
    def __init__(self, token_name):
        super().__init__(f"token {token_name} is not expired")


@patch_to(DJ_WorkspaceAccount)
async def regenerate_access_token(
    self,
    domo_user: dmdu.DomoUser,
    auth: dmda.DomoAuth,
    token_name: str,
    duration_in_days: int = 90,
    access_token_days_to_expiry_threshold: int = 7,
    is_force_regeneration: bool = False,
    raise_exception: bool = True,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
):

    domo_config = dmic.DomoInstanceConfig(auth=auth)

    if not is_force_regeneration:
        domo_access_tokens = await domo_config.get_access_tokens(
            debug_api=debug_api, session=session
        )

        domo_access_token = next(
            (
                token
                for token in domo_access_tokens
                if token.owner == domo_user and token.name == token_name
            ),
            None,
        )

        if (
            domo_access_token
            and access_token_days_to_expiry_threshold
            <= (domo_access_token.expiration_date - dt.datetime.now()).days
        ):
            if raise_exception:
                raise AccessToken_NotExpired(token_name)

            return domo_access_token

    domo_access_token = await domo_config.regenerate_access_token(
        domo_user=domo_user,
        token_name=token_name,
        session=session,
        duration_in_days=duration_in_days,
        debug_api=debug_api,
    )
    self.domo_access_token = domo_access_token.token

    return domo_access_token


@patch_to(DJ_WorkspaceAccount)
async def regenerate_target_access_token(
    self,
    token_name: str = None,
    duration_in_days: int = 90,
    access_token_days_to_expiry_threshold: int = 7,
    is_force_regeneration: bool = False,
    debug_api: bool = False,
    trigger_upsert_accounts: bool = True,
    target_account_name: str = None,
    target_auth_backup: dmda.DomoAuth = None,
    remote_account_name: str = None,
    remote_auth_ls: List[dmda.DomoAuth] = None,
):

    if not self.is_valid_token_auth:
        is_force_regeneration = True

    target_auth = await self.generate_target_auth(backup_auth=target_auth_backup)

    domo_user = await dmdu.DomoUsers.by_email(
        email_ls=[self.username], auth=target_auth
    )

    domo_access_token = await self.regenerate_access_token(
        domo_user=domo_user,
        auth=target_auth,
        token_name=token_name or self.local_account_name,
        duration_in_days=duration_in_days,
        access_token_days_to_expiry_threshold=access_token_days_to_expiry_threshold,
        is_force_regeneration=is_force_regeneration,
        raise_exception=False,
        debug_api=debug_api,
    )

    if trigger_upsert_accounts:
        return await self.upsert_accounts(
            target_account_name=target_account_name or self.local_account_name,
            target_auth_backup=target_auth_backup,
            remote_account_name=remote_account_name or self.local_account_name,
            remote_auth_ls=remote_auth_ls,
            debug_api=debug_api,
        )

    return domo_access_token



"""Fill in a module description here"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/LocalAccount/ConfigInstanceAccount.ipynb.

# %% auto 0
__all__ = ['LocalAccount', 'generate_config_instance_accounts_from_df', 'LocalAccountConfig']

# %% ../nbs/LocalAccount/ConfigInstanceAccount.ipynb 2
from dataclasses import dataclass, field
from typing import List

import domolibrary.client.DomoAuth as dmda
import domolibrary.classes.DomoAccount as dmac

import domolocal.DJ_Workspace as dlws
import domolocal.DJ_WorkspaceAccount as dlac
import domolocal.core as dlcr
import pandas as pd

# %% ../nbs/LocalAccount/ConfigInstanceAccount.ipynb 4
@dataclass
class LocalAccount(dlac.DJ_WorkspaceAccount):
    target_account_name: str = None

    local_auth: dmda.DomoAuth = field(repr=False, default=None)

    dj_account: dlac.DJ_WorkspaceAccount = field(repr=False, default=None)

    is_shared_with_workspace: bool = False
    is_backup_account_valid: bool = False

    testing_err: str = None

    def to_json(self):

        obj = {
            "target_instance": self.target_instance,
            "remote_account_name": self.local_account_name,
            "is_domo_account_exists": self.is_domo_account_exists,
            "is_shared_with_workspace": self.is_shared_with_workspace,
            "is_backup_account_valid": self.is_backup_account_valid,
            "is_valid_full_auth": self.is_valid_full_auth,
            "is_valid_token_auth": self.is_valid_token_auth,
            "err": self.testing_err,
            "target_account_name": self.target_account_name,
        }

        return obj

    @staticmethod
    def _match_domo_account(account_name, domo_accounts: List[dmac.DomoAccount]):
        return next(
            (
                domo_account
                for domo_account in domo_accounts
                if domo_account.name == account_name
            ),
            None,
        )

    @classmethod
    def generate_config_admin_from_instance(
        cls,
        target_instance: str,
        local_auth: dmda.DomoAuth,
        existing_domo_accounts: List[dmac.DomoAccounts],
    ):

        remote_account_name = f"sdk_{target_instance}"
        target_account_name = remote_account_name

        domo_account = cls._match_domo_account(
            account_name=remote_account_name, domo_accounts=existing_domo_accounts
        )

        config = cls(
            remote_account_name=remote_account_name,
            target_account_name=target_account_name,
            target_instance=target_instance,
            local_auth=local_auth,
        )

        config.set_domo_account(domo_account)

        return config

# %% ../nbs/LocalAccount/ConfigInstanceAccount.ipynb 5
async def generate_config_instance_accounts_from_df(
    config_df: pd.DataFrame, local_auth: dmda.DomoAuth
) -> List[LocalAccount]:

    existing_domo_accounts = await dmac.DomoAccounts.get_accounts(local_auth)

    return [
        LocalAccount.generate_config_admin_from_instance(
            local_auth=local_auth,
            target_instance=obj["domo_instance"],
            existing_domo_accounts=existing_domo_accounts,
        )
        for obj in config_df.to_dict(orient="records")
    ]

# %% ../nbs/LocalAccount/ConfigInstanceAccount.ipynb 7
@dataclass
class LocalAccountConfig(DJA_WorkspaceAccount):
    pass


#     target_instance: str
#     dj_workspace: Any = field(repr=False) #DJ_Workspace

#     local_account_name: str

#     local_creds: dict = field(repr=False, default=None)

#     is_valid_full_auth: bool = None
#     is_valid_token_auth: bool = None

#     _target_token_auth: dmda.DomoAuth = None
#     _target_full_auth: dmda.DomoAuth = None
#     target_auth : dmda.DomoAuth = None


#     domo_username: str = None
#     domo_password: str = field(repr=False, default=None)
#     domo_access_token: str = field(repr=False, default=None)

#     account_config : dmacc.AccountConfig = None

#     def __eq__(self, other):
#         if self.__class__.__name__ != other.__class__.__name__:
#             return False

#         return self.local_account_name == other.local_account_name and self.dj_workspace == other.dj_workspace

#     def __lt__(self, other):
#          return self.local_account_name < other.local_account_names

#     def to_json(self):
#         return {
#             "target_instance": self.target_instance,
#             "local_account_name": self.local_account_name,
#             "is_valid_full_auth": self.is_valid_full_auth,
#             "is_valid_token_auth": self.is_valid_token_auth,
#         }

# @classmethod
# def from_workspace_by_account_name(
#     cls,
#     domo_instance: str,
#     local_account_name : str,
#     dj_workspace: Any,
# ):
#     return cls(
#         target_instance=domo_instance,
#         local_account_name=local_account_name,
#         dj_workspace=dj_workspace,
#     )

# %% ../nbs/LocalAccount/ConfigInstanceAccount.ipynb 9
@patch_to(LocalAccountConfig)
async def get_local_creds(self, is_abstract=False):
    """
    attemps to read local creds from workspace by account_name.
    will attempt to share account with workspace
    will throw an exception if account cannot be read from workspace
    """

    retry = 0

    while retry <= 1 and not self.local_creds:
        try:
            creds = dlcr.read_domo_jupyter_account(
                self.local_account_name,
                domojupyter_fn=self.dj_workspace.domojupyter_fn,
                is_abstract=is_abstract,
            )
            self.local_creds = creds
            return self.local_creds

        except (dmde.DomoError, dlcr.DJ_Workspace_NoAuth) as e:
            if retry >= 1:
                self.dj_workspace.fix_no_account.append(self)
                self.is_valid_token_auth = False
                self.is_valid_full_auth = False
                return None

            await self.dj_workspace.add_account_to_jupyter_workspace(
                account_name=self.local_account_name,
                share_group_name=self.dj_workspace.default_share_group_name,
                config=self,
            )
            retry += 1


@patch_to(LocalAccountConfig)
async def test_local_creds_full_auth(
    self,
    skip_retrieve_creds: bool = True,
    debug_prn: bool = False,
):
    """
    validates local_creds have valid credentials
    will update "fix_reset_password" task list in dj_workspace if invalid creds
    """

    if not skip_retrieve_creds or not self.domo_username or not self.domo_password:
        await self.get_local_creds()

        self.domo_username = self.local_creds.get("username")
        self.domo_password = self.local_creds.get("password")

    if not self.domo_username:
        raise LA_NoUsername(self.local_account_name)

    if not self.domo_password:
        raise LA_NoPassword(self.local_account_name)

    target_full_auth = dmda.DomoFullAuth(
        domo_instance=self.target_instance,
        domo_username=self.domo_username,
        domo_password=self.domo_password,
    )

    self._target_full_auth = target_full_auth

    try:
        await target_full_auth.print_is_token()
        self.is_valid_full_auth = True

    except dmde.DomoError as e:
        dlut.printmd(
            f"🤯 test_full_auth for: ***{self.local_account_name}*** returned {e}"
        )

        self.is_valid_full_auth = False
        self.dj_workspace.fix_reset_password.append(self)

    return self.is_valid_full_auth

# %% ../nbs/LocalAccount/ConfigInstanceAccount.ipynb 11
@patch_to(LocalAccountConfig)
async def test_local_creds_token_auth(
    self,
    skip_retrieve_creds: bool = False,
    debug_prn: bool = False,
):

    if not skip_retrieve_creds or not self.domo_access_token:
        await self.get_local_creds()

        self.domo_access_token = self.local_creds.get("domoAccessToken")

    if not self.domo_access_token:
        raise LA_NoAccessToken(self.local_account_name)

    target_token_auth = dmda.DomoTokenAuth(
        domo_instance=self.target_instance, domo_access_token=self.domo_access_token
    )

    self._target_token_auth = target_token_auth

    try:
        await target_token_auth.print_is_token()
        self.is_valid_token_auth = True

    except dmde.DomoError as e:
        dlut.printmd(
            f"🤯 test_token_auth for: ***{self.local_account_name}*** returned {e}"
        )
        self.is_valid_token_auth = False
        self.dj_workspace.fix_regenerate_access_token.append(self)

    return self.is_valid_token_auth


@patch_to(LocalAccountConfig)
async def test_local_creds(self):
    """test local token auth and local full auth"""
    return all(
        (
            await dmce.gather_with_concurrency(
                *[
                    self.test_local_creds_full_auth(),
                    self.test_local_creds_token_auth(),
                ],
                n=10,
            )
        )
    )


@patch_to(LocalAccountConfig)
async def generate_target_auth(self, backup_auth: dmda.DomoAuth = None):
    target_auth = None

    if self.is_valid_token_auth:
        target_auth = self._target_token_auth

    if self.is_valid_full_auth:
        target_auth = self._target_full_auth

    if not target_auth:
        target_auth = backup_auth

    if not target_auth:
        raise LA_ValidAuth(
            domo_instance=self.target_instance, account_name=self.local_account_name
        )

    await target_auth.print_is_token()

    self.target_auth = target_auth

    return self.target_auth

# %% ../nbs/LocalAccount/ConfigInstanceAccount.ipynb 13
@patch_to(LocalAccountConfig)
async def _search_domo_groups_by_name(
    self, auth: dmda.DomoAuth, group_names: List[str]
):
    await dmdg.DomoGroups.toggle_system_group_visibility(
        is_hide_system_groups=False, auth=auth
    )

    await asyncio.sleep(1)

    domo_groups = await dmce.gather_with_concurrency(
        *[
            dmdg.DomoGroup.search_by_name(group_name=group_name, auth=auth)
            for group_name in group_names
        ],
        n=10,
    )
    await dmdg.DomoGroups.toggle_system_group_visibility(
        is_hide_system_groups=True, auth=auth
    )
    return domo_groups


@patch_to(LocalAccountConfig)
async def _upsert_group(
    self,
    auth: dmda.DomoAuth,
    group_name: str,
    group_owner_names: list[str] = [
        "Role: sie_instance_admin",
        "Role: sie_dde_instance_admin",
    ],
    debug_api: bool = False,
):

    domo_group = await dmdg.DomoGroup.upsert(
        group_name=group_name,
        group_type=dmdg.GroupType_Enum["CLOSED"].value,
        description=f"updated via {dt.date.today()}",
        auth=auth,
        debug_api=debug_api,
    )

    domo_group_owners = await self._search_domo_groups_by_name(
        auth=auth, group_names=group_owner_names
    )

    await domo_group.Membership.add_owners(add_owner_ls=domo_group_owners)

    return domo_group


@patch_to(LocalAccountConfig)
def _generate_account_config(self):
    account_config = dmacc.AccountConfig.domo_access_token.value(
        username=self.domo_username,
        password=self.domo_password,
        domo_access_token=self.domo_access_token,
    )

    return account_config


@patch_to(LocalAccountConfig)
async def _upsert_account_and_share_with_default_group(
    self,
    account_config: dmacc.AccountConfig,
    account_name: str,
    auth: dmda.DomoAuth,
    debug_api: bool = False,
    default_share_group: dmdg.DomoGroup = None,
):

    domo_account = await dmacc.DomoAccounts.upsert_account(
        account_config=account_config,
        account_name=account_name,
        auth=auth,
        debug_api=debug_api,
    )

    if default_share_group:
        share_res = await domo_account._share_v2(group_id=default_share_group.id)

    return domo_account


@patch_to(LocalAccountConfig)
async def _upsert_account(
    self,
    auth: dmda.DomoAuth,
    account_name: str,
    default_share_group_name: str = None,
    default_share_group_owner_names=[
        "Role: sie_instance_admin",
        "Role: sie_dde_instance_admin",
    ],
    debug_api: bool = False,
    is_test: bool = False,
):
    account_config = self._generate_account_config()

    group_name = default_share_group_name or self.dj_workspace.default_share_group_name

    default_share_group = None

    if group_name:
        default_share_group = await self._upsert_group(
            auth=auth,
            group_name=group_name,
            group_owner_names=default_share_group_owner_names
            or self.default_share_group_name,
            debug_api=debug_api,
        )

    if is_test:
        return account_config, default_share_group

    domo_account = await self._upsert_account_and_share_with_default_group(
        auth=auth,
        account_config=account_config,
        account_name=account_name,
        default_share_group=default_share_group,
        debug_api=debug_api,
    )

    return domo_account


@patch_to(LocalAccountConfig)
async def upsert_local_account(self, is_test=True, debug_api: bool = False):
    try:
        return await self._upsert_account(
            auth=self.dj_workspace.local_auth,
            account_name=self.local_account_name,
            is_test=is_test,
            debug_api=debug_api,
        )
    except dmde.DomoError as e:
        print(e)
        return None


@patch_to(LocalAccountConfig)
async def upsert_target_account(
    self,
    target_account_name: str,
    backup_auth: dmda.DomoAuth = None,
    is_test=True,
    debug_api: bool = False,
):
    try:
        target_auth = await self.generate_target_auth(backup_auth=backup_auth)
    except LA_ValidAuth as e:
        raise LA_UpdateCredsError(
            cred_type="target_account",
            local_account_name=target_account_name,
            additional_message=str(e),
        ) from e

        return await self._upsert_account(
            auth=target_auth,
            account_name=target_account_name,
            is_test=is_test,
            debug_api=debug_api,
        )

    except dmde.DomoError as e:
        print(e)
        return None


@patch_to(LocalAccountConfig)
async def upsert_remote_account(
    self,
    remote_account_name: str,
    auth: dmda.DomoAuth,
    is_test=True,
    debug_api: bool = False,
):
    try:
        return await self._upsert_account(
            auth=auth,
            account_name=remote_account_name,
            is_test=is_test,
            debug_api=debug_api,
        )
    except dmde.DomoError as e:
        print(e)
        return None


@patch_to(LocalAccountConfig)
async def upsert_accounts(
    self,
    target_account_name: str = None,
    target_auth_backup: dmda.DomoAuth = None,
    remote_account_name: str = None,
    remote_auth_ls: List[dmda.DomoAuth] = None,
    debug_api: bool = False,
):
    """attempts to upsert account in target instance AND deploy account to remote instances"""
    target_acc = await self.upsert_target_account(
        target_account_name=target_account_name or self.local_account_name,
        backup_auth=target_auth_backup,
        is_test=False,
        debug_api=debug_api,
    )

    local_acc = await self.upsert_local_account(is_test=False, debug_api=debug_api)

    remote_accs = None
    if remote_auth_ls:
        remote_accs = await dmce.gather_with_concurrency(
            *[
                self.upsert_remote_account(
                    remote_account_name=remote_account_name or self.local_account_name,
                    auth=r_auth,
                )
                for r_auth in remote_auth_ls
            ],
            n=10,
        )

    return {"target": target_acc, "local": local_acc, "remotes": remote_accs}

# %% ../nbs/LocalAccount/ConfigInstanceAccount.ipynb 16
class AccessToken_NotExpired(dlcr.ClassError):
    def __init__(self, token_name):
        super().__init__(f"token {token_name} is not expired")


@patch_to(LocalAccountConfig)
async def regenerate_access_token(
    self,
    domo_user: dmdu.DomoUser,
    auth: dmda.DomoAuth,
    token_name: str,
    duration_in_days: int = 90,
    access_token_days_to_expiry_threshold: int = 7,
    is_force_regeneration: bool = False,
    raise_exception: bool = True,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
):

    domo_config = dmic.DomoInstanceConfig(auth=auth)

    if not is_force_regeneration:
        domo_access_tokens = await domo_config.get_access_tokens(
            debug_api=debug_api, session=session
        )

        domo_access_token = next(
            (
                token
                for token in domo_access_tokens
                if token.owner == domo_user and token.name == token_name
            ),
            None,
        )

        if (
            domo_access_token
            and access_token_days_to_expiry_threshold
            <= (domo_access_token.expiration_date - dt.datetime.now()).days
        ):
            if raise_exception:
                raise AccessToken_NotExpired(token_name)

            return domo_access_token

    domo_access_token = await domo_config.regenerate_access_token(
        domo_user=domo_user,
        token_name=token_name,
        session=session,
        duration_in_days=duration_in_days,
        debug_api=debug_api,
    )
    self.domo_access_token = domo_access_token.token

    return domo_access_token


@patch_to(LocalAccountConfig)
async def regenerate_target_access_token(
    self,
    token_name: str = None,
    duration_in_days: int = 90,
    access_token_days_to_expiry_threshold: int = 7,
    is_force_regeneration: bool = False,
    debug_api: bool = False,
    trigger_upsert_accounts: bool = True,
    target_account_name: str = None,
    target_auth_backup: dmda.DomoAuth = None,
    remote_account_name: str = None,
    remote_auth_ls: List[dmda.DomoAuth] = None,
):

    if not self.is_valid_token_auth:
        is_force_regeneration = True

    target_auth = await self.generate_target_auth(backup_auth=target_auth_backup)

    domo_user = await dmdu.DomoUsers.by_email(
        email_ls=[self.domo_username], auth=target_auth
    )

    domo_access_token = await self.regenerate_access_token(
        domo_user=domo_user,
        auth=target_auth,
        token_name=token_name or self.local_account_name,
        duration_in_days=duration_in_days,
        access_token_days_to_expiry_threshold=access_token_days_to_expiry_threshold,
        is_force_regeneration=is_force_regeneration,
        raise_exception=False,
        debug_api=debug_api,
    )

    if trigger_upsert_accounts:
        return await self.upsert_accounts(
            target_account_name=target_account_name or self.local_account_name,
            target_auth_backup=target_auth_backup,
            remote_account_name=remote_account_name or self.local_account_name,
            remote_auth_ls=remote_auth_ls,
            debug_api=debug_api,
        )

    return domo_access_token



"""Fill in a module description here"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/LocalAccount/00_DJ_Workspace.ipynb.

# %% ../nbs/LocalAccount/00_DJ_Workspace.ipynb 2
from __future__ import annotations
from typing import List, Callable
from dataclasses import dataclass, field
import datetime as dt
import pandas as pd

import asyncio

import domolocal.core as dlcr

import domolibrary.client.DomoAuth as dmda
import domolibrary.client.DomoError as dmde
import domolibrary.classes.DomoGroup as dmdg

import domolibrary.utils.chunk_execution as dmce

import domolibrary.classes.DomoJupyter as dmdj
import domolocal.domolibrary_utils as dldm
from nbdev.showdoc import patch_to

# %% auto 0
__all__ = ['DJW_NoAccount', 'DJW_NoGroup', 'DJ_Workspace']

# %% ../nbs/LocalAccount/00_DJ_Workspace.ipynb 3
from domolocal.utils import generate_domo_password

# %% ../nbs/LocalAccount/00_DJ_Workspace.ipynb 7
class DJW_NoAccount(dlcr.ClassError):
    def __init__(self, account_name, domo_instance):
        super().__init__(
            f"unable to retrieve account - {account_name} from {domo_instance}"
        )


class DJW_NoGroup(dlcr.ClassError):
    def __init__(self, group_name, domo_instance):
        super().__init__(
            f"unable to retrieve group - {group_name} from {domo_instance}"
        )

# %% ../nbs/LocalAccount/00_DJ_Workspace.ipynb 8
@dataclass
class DJ_Workspace:
    name: str
    local_auth: dmda.DomoAuth = field(repr=False)
    domo_workspace: dmdj.DomoJupyterWorkspace = field(repr=False)

    domojupyter_fn: Callable = field(repr=False)

    generate_password_fn: Callable = field(repr=False, default=None)

    default_share_group_name: str = None
    default_share_group_owner_names: list[str] = field(
        default_factory=lambda: [
            "Role: sie_instance_admin",
            "Role: sie_dde_instance_admin",
        ]
    )

    dj_accounts: List[Any] = field(repr=False, default_factory=list)

    fix_no_account: List[str] = field(default_factory=list)
    fix_reset_password: List[str] = field(default_factory=list)
    fix_regenerate_access_token: List[str] = field(default_factory=list)

    def __eq__(self, other):
        if self.__class__.__name__ != other.__class__.__name__:
            return False

        return (
            self.name == other.name
            and self.local_auth.domo_instance == other.local_auth.domo_instance
        )

    @classmethod
    async def from_workspace_name(
        cls,
        auth: dmda.DomoAuth,
        workspace_name: str,
        domojupyter_fn: Callable,
        generate_password_fn: Callable = None,
        default_share_group_name: str = None,
        default_share_group_owner_names: list[str] = None,
    ):

        default_share_group_owner_names = default_share_group_owner_names or [
            "Role: sie_instance_admin",
            "Role: sie_dde_instance_admin",
        ]

        domo_workspace = await dmdj.search_workspace_by_name(
            auth=auth, workspace_name=workspace_name
        )

        return cls(
            name=workspace_name,
            local_auth=auth,
            domo_workspace=domo_workspace,
            domojupyter_fn=domojupyter_fn,
            generate_password_fn=generate_password_fn,
            default_share_group_name=default_share_group_name,
            default_share_group_owner_names=default_share_group_owner_names,
        )

# %% ../nbs/LocalAccount/00_DJ_Workspace.ipynb 10
@patch_to(DJ_Workspace)
async def add_account_to_jupyter_workspace(
    self, account_name, share_group_name=None, config=None
):
    try:
        domo_account = await dldm.search_domo_account_by_name(
            auth=self.local_auth, account_name=account_name
        )

        share_group_name = share_group_name or self.default_share_group_name

        share_domo_group = await dldm.search_or_upsert_domo_group(
            auth=self.local_auth, group_name=share_group_name, upsert_if_not_exist=True
        )

        await domo_account._share_v2(group_id=share_domo_group.id, auth=self.local_auth)

        await self.domo_workspace.add_account(
            domo_account=domo_account, update_config=False
        )

        return await self.domo_workspace.update_config()

    except (dmde.DomoError, DJW_NoAccount) as e:
        self.dj_workspace.fix_no_account.append(config)


@patch_to(DJ_Workspace)
def generate_config_from_domo_instace(self, domo_instance, account_name=None):

    import domolocal.DJ_WorkspaceAccount as dlwa

    try:
        account_name = account_name or f"sdk_{domo_instance}"
        return dlwa.DJ_WorkspaceAccount.from_workspace_by_account_name(
            domo_instance=domo_instance,
            local_account_name=account_name,
            dj_workspace=self,
        )

    except DJW_NoAccount as e:
        print(f"🤯 - {e} - {domo_instance}")
        self.fix_no_account.append(domo_instance)
        return None

# %% ../nbs/LocalAccount/00_DJ_Workspace.ipynb 12
@patch_to(DJ_Workspace)
def generate_configs_from_domo_instances(
    self: DJ_Workspace, domo_instance_ls: List[str]
):

    self.local_configs = [
        self.generate_config_from_domo_instace(domo_instance=domo_instance)
        for domo_instance in domo_instance_ls
    ]

    return self.local_configs

# %% ../nbs/LocalAccount/00_DJ_Workspace.ipynb 14
class DJW_NoConfigs(dlcr.ClassError):
    def __init__(self):
        super().__init__("no configs in self.local_configs.  Run generate_configs")


@patch_to(DJ_Workspace)
async def generate_fix_report(self) -> pd.DataFrame:
    if not self.local_configs:
        raise DJW_NoConfigs()

    for config in self.local_configs:
        try:
            await config.test_local_creds()
            if not config.is_valid_full_auth:
                self.fix_password.append(config.domo_instance)
            if not config.is_valid_token_auth:
                self.regenerate_access_token.append(config.domo_instance)

        except (DJW_NoAccount, Exception) as e:
            self.fix_no_account.append(config)

    df = [
        *[
            {"domo_instance": domo_instance, "task": "no_account"}
            for domo_instance in self.fix_no_account
            if domo_instance
        ],
        *[
            {"domo_instance": la.target_instance, "task": "reset_password"}
            for la in self.fix_reset_password
            if la
        ],
        *[
            {
                "domo_instance": la.target_instance,
                "task": "regenerate_access_token",
            }
            for la in self.fix_regenerate_access_token
            if la
        ],
    ]

    return {"num_errors_to_fix": len(df), "report": df}



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'gh-page',
                'doc_baseurl': '/domolibrary_execution',
                'doc_host': 'https://.github.io',
                'git_url': 'https://github.com//domolibrary_execution',
                'lib_path': 'domolibrary_execution'},
  'syms': { 'domolibrary_execution.ConfigInstanceAccount': { 'domolibrary_execution.ConfigInstanceAccount.AccessToken_NotExpired': ( 'LocalAccount/configinstanceaccount.html#accesstoken_notexpired',
                                                                                                                                     'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.AccessToken_NotExpired.__init__': ( 'LocalAccount/configinstanceaccount.html#accesstoken_notexpired.__init__',
                                                                                                                                              'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccount': ( 'LocalAccount/configinstanceaccount.html#localaccount',
                                                                                                                           'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccount._match_domo_account': ( 'LocalAccount/configinstanceaccount.html#localaccount._match_domo_account',
                                                                                                                                               'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccount.generate_config_admin_from_instance': ( 'LocalAccount/configinstanceaccount.html#localaccount.generate_config_admin_from_instance',
                                                                                                                                                               'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccount.to_json': ( 'LocalAccount/configinstanceaccount.html#localaccount.to_json',
                                                                                                                                   'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig',
                                                                                                                                 'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig._generate_account_config': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig._generate_account_config',
                                                                                                                                                          'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig._search_domo_groups_by_name': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig._search_domo_groups_by_name',
                                                                                                                                                             'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig._upsert_account': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig._upsert_account',
                                                                                                                                                 'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig._upsert_account_and_share_with_default_group': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig._upsert_account_and_share_with_default_group',
                                                                                                                                                                              'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig._upsert_group': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig._upsert_group',
                                                                                                                                               'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig.generate_target_auth': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig.generate_target_auth',
                                                                                                                                                      'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig.get_local_creds': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig.get_local_creds',
                                                                                                                                                 'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig.regenerate_access_token': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig.regenerate_access_token',
                                                                                                                                                         'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig.regenerate_target_access_token': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig.regenerate_target_access_token',
                                                                                                                                                                'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig.test_local_creds': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig.test_local_creds',
                                                                                                                                                  'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig.test_local_creds_full_auth': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig.test_local_creds_full_auth',
                                                                                                                                                            'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig.test_local_creds_token_auth': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig.test_local_creds_token_auth',
                                                                                                                                                             'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig.upsert_accounts': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig.upsert_accounts',
                                                                                                                                                 'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig.upsert_local_account': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig.upsert_local_account',
                                                                                                                                                      'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig.upsert_remote_account': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig.upsert_remote_account',
                                                                                                                                                       'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.LocalAccountConfig.upsert_target_account': ( 'LocalAccount/configinstanceaccount.html#localaccountconfig.upsert_target_account',
                                                                                                                                                       'domolibrary_execution/ConfigInstanceAccount.py'),
                                                             'domolibrary_execution.ConfigInstanceAccount.generate_config_instance_accounts_from_df': ( 'LocalAccount/configinstanceaccount.html#generate_config_instance_accounts_from_df',
                                                                                                                                                        'domolibrary_execution/ConfigInstanceAccount.py')},
            'domolibrary_execution.DJ_Workspace': { 'domolibrary_execution.DJ_Workspace.DJW_NoAccount': ( 'LocalAccount/dj_workspace.html#djw_noaccount',
                                                                                                          'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJW_NoAccount.__init__': ( 'LocalAccount/dj_workspace.html#djw_noaccount.__init__',
                                                                                                                   'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJW_NoConfigs': ( 'LocalAccount/dj_workspace.html#djw_noconfigs',
                                                                                                          'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJW_NoConfigs.__init__': ( 'LocalAccount/dj_workspace.html#djw_noconfigs.__init__',
                                                                                                                   'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJW_NoGroup': ( 'LocalAccount/dj_workspace.html#djw_nogroup',
                                                                                                        'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJW_NoGroup.__init__': ( 'LocalAccount/dj_workspace.html#djw_nogroup.__init__',
                                                                                                                 'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJ_Workspace': ( 'LocalAccount/dj_workspace.html#dj_workspace',
                                                                                                         'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJ_Workspace.__eq__': ( 'LocalAccount/dj_workspace.html#dj_workspace.__eq__',
                                                                                                                'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJ_Workspace.add_account_to_jupyter_workspace': ( 'LocalAccount/dj_workspace.html#dj_workspace.add_account_to_jupyter_workspace',
                                                                                                                                          'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJ_Workspace.from_workspace_name': ( 'LocalAccount/dj_workspace.html#dj_workspace.from_workspace_name',
                                                                                                                             'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJ_Workspace.generate_config_from_domo_instace': ( 'LocalAccount/dj_workspace.html#dj_workspace.generate_config_from_domo_instace',
                                                                                                                                           'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJ_Workspace.generate_configs_from_domo_instances': ( 'LocalAccount/dj_workspace.html#dj_workspace.generate_configs_from_domo_instances',
                                                                                                                                              'domolibrary_execution/DJ_Workspace.py'),
                                                    'domolibrary_execution.DJ_Workspace.DJ_Workspace.generate_fix_report': ( 'LocalAccount/dj_workspace.html#dj_workspace.generate_fix_report',
                                                                                                                             'domolibrary_execution/DJ_Workspace.py')},
            'domolibrary_execution.DJ_WorkspaceAccount': { 'domolibrary_execution.DJ_WorkspaceAccount.AccessToken_NotExpired': ( 'LocalAccount/dj_workspaceaccount.html#accesstoken_notexpired',
                                                                                                                                 'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.AccessToken_NotExpired.__init__': ( 'LocalAccount/dj_workspaceaccount.html#accesstoken_notexpired.__init__',
                                                                                                                                          'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJA_UpdateCredsError': ( 'LocalAccount/dj_workspaceaccount.html#dja_updatecredserror',
                                                                                                                               'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJA_UpdateCredsError.__init__': ( 'LocalAccount/dj_workspaceaccount.html#dja_updatecredserror.__init__',
                                                                                                                                        'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount',
                                                                                                                              'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount._generate_account_config': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount._generate_account_config',
                                                                                                                                                       'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount._search_domo_groups_by_name': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount._search_domo_groups_by_name',
                                                                                                                                                          'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount._upsert_account': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount._upsert_account',
                                                                                                                                              'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount._upsert_account_and_share_with_default_group': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount._upsert_account_and_share_with_default_group',
                                                                                                                                                                           'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount._upsert_group': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount._upsert_group',
                                                                                                                                            'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount.from_workspace_by_account_name': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount.from_workspace_by_account_name',
                                                                                                                                                             'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount.regenerate_access_token': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount.regenerate_access_token',
                                                                                                                                                      'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount.regenerate_target_access_token': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount.regenerate_target_access_token',
                                                                                                                                                             'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount.upsert_accounts': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount.upsert_accounts',
                                                                                                                                              'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount.upsert_local_account': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount.upsert_local_account',
                                                                                                                                                   'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount.upsert_remote_account': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount.upsert_remote_account',
                                                                                                                                                    'domolibrary_execution/DJ_WorkspaceAccount.py'),
                                                           'domolibrary_execution.DJ_WorkspaceAccount.DJ_WorkspaceAccount.upsert_target_account': ( 'LocalAccount/dj_workspaceaccount.html#dj_workspaceaccount.upsert_target_account',
                                                                                                                                                    'domolibrary_execution/DJ_WorkspaceAccount.py')},
            'domolibrary_execution.FactoryAccount': { 'domolibrary_execution.FactoryAccount.Config_UpsertDomoAccessTokenAccount': ( 'factory/factoryaccount.html#config_upsertdomoaccesstokenaccount',
                                                                                                                                    'domolibrary_execution/FactoryAccount.py'),
                                                      'domolibrary_execution.FactoryAccount.modify_generate_destination_domo_access_token_config_fn': ( 'factory/factoryaccount.html#modify_generate_destination_domo_access_token_config_fn',
                                                                                                                                                        'domolibrary_execution/FactoryAccount.py'),
                                                      'domolibrary_execution.FactoryAccount.modify_generate_domo_access_token_config_fn': ( 'factory/factoryaccount.html#modify_generate_domo_access_token_config_fn',
                                                                                                                                            'domolibrary_execution/FactoryAccount.py'),
                                                      'domolibrary_execution.FactoryAccount.upsert_domoaccount': ( 'factory/factoryaccount.html#upsert_domoaccount',
                                                                                                                   'domolibrary_execution/FactoryAccount.py')},
            'domolibrary_execution.FactoryAuth': { 'domolibrary_execution.FactoryAuth.Config_DomoJupyterToDomoAccount': ( 'factory/factoryauth.html#config_domojupytertodomoaccount',
                                                                                                                          'domolibrary_execution/FactoryAuth.py'),
                                                   'domolibrary_execution.FactoryAuth.Config_GenerateConfigAuth': ( 'factory/factoryauth.html#config_generateconfigauth',
                                                                                                                    'domolibrary_execution/FactoryAuth.py'),
                                                   'domolibrary_execution.FactoryAuth.generate_config_auth': ( 'factory/factoryauth.html#generate_config_auth',
                                                                                                               'domolibrary_execution/FactoryAuth.py'),
                                                   'domolibrary_execution.FactoryAuth.get_jupyteraccount': ( 'factory/factoryauth.html#get_jupyteraccount',
                                                                                                             'domolibrary_execution/FactoryAuth.py'),
                                                   'domolibrary_execution.FactoryAuth.modify_config_generate_admin_auth_fn': ( 'factory/factoryauth.html#modify_config_generate_admin_auth_fn',
                                                                                                                               'domolibrary_execution/FactoryAuth.py'),
                                                   'domolibrary_execution.FactoryAuth.modify_config_generate_config_auth_fn': ( 'factory/factoryauth.html#modify_config_generate_config_auth_fn',
                                                                                                                                'domolibrary_execution/FactoryAuth.py')},
            'domolibrary_execution.FactoryRole': { 'domolibrary_execution.FactoryRole.GetRole': ( 'factory/factoryrole.html#getrole',
                                                                                                  'domolibrary_execution/FactoryRole.py'),
                                                   'domolibrary_execution.FactoryRole.UpsertRoleConfig': ( 'factory/factoryrole.html#upsertroleconfig',
                                                                                                           'domolibrary_execution/FactoryRole.py'),
                                                   'domolibrary_execution.FactoryRole.get_role': ( 'factory/factoryrole.html#get_role',
                                                                                                   'domolibrary_execution/FactoryRole.py'),
                                                   'domolibrary_execution.FactoryRole.upsert_role': ( 'factory/factoryrole.html#upsert_role',
                                                                                                      'domolibrary_execution/FactoryRole.py')},
            'domolibrary_execution.FactoryUser': { 'domolibrary_execution.FactoryUser.DeletetUser_Config': ( 'factory/factoryuser.html#deletetuser_config',
                                                                                                             'domolibrary_execution/FactoryUser.py'),
                                                   'domolibrary_execution.FactoryUser.GetUser_Config': ( 'factory/factoryuser.html#getuser_config',
                                                                                                         'domolibrary_execution/FactoryUser.py'),
                                                   'domolibrary_execution.FactoryUser.RegenerateUserAccessToken_Config': ( 'factory/factoryuser.html#regenerateuseraccesstoken_config',
                                                                                                                           'domolibrary_execution/FactoryUser.py'),
                                                   'domolibrary_execution.FactoryUser.UpsertUser_Config': ( 'factory/factoryuser.html#upsertuser_config',
                                                                                                            'domolibrary_execution/FactoryUser.py'),
                                                   'domolibrary_execution.FactoryUser.delete_user': ( 'factory/factoryuser.html#delete_user',
                                                                                                      'domolibrary_execution/FactoryUser.py'),
                                                   'domolibrary_execution.FactoryUser.get_user': ( 'factory/factoryuser.html#get_user',
                                                                                                   'domolibrary_execution/FactoryUser.py'),
                                                   'domolibrary_execution.FactoryUser.modify_config_calc_domo_role': ( 'factory/factoryuser.html#modify_config_calc_domo_role',
                                                                                                                       'domolibrary_execution/FactoryUser.py'),
                                                   'domolibrary_execution.FactoryUser.modify_config_is_force_regenerate_based_on_expiration_threshold_fn': ( 'factory/factoryuser.html#modify_config_is_force_regenerate_based_on_expiration_threshold_fn',
                                                                                                                                                             'domolibrary_execution/FactoryUser.py'),
                                                   'domolibrary_execution.FactoryUser.modify_config_user_email_for_upsert_user_fn': ( 'factory/factoryuser.html#modify_config_user_email_for_upsert_user_fn',
                                                                                                                                      'domolibrary_execution/FactoryUser.py'),
                                                   'domolibrary_execution.FactoryUser.post_config_fail_if_no_token': ( 'factory/factoryuser.html#post_config_fail_if_no_token',
                                                                                                                       'domolibrary_execution/FactoryUser.py'),
                                                   'domolibrary_execution.FactoryUser.regenerate_user_accesstoken': ( 'factory/factoryuser.html#regenerate_user_accesstoken',
                                                                                                                      'domolibrary_execution/FactoryUser.py'),
                                                   'domolibrary_execution.FactoryUser.upsert_user': ( 'factory/factoryuser.html#upsert_user',
                                                                                                      'domolibrary_execution/FactoryUser.py')},
            'domolibrary_execution.LocalAccount.shortcut_fn': { 'domolibrary_execution.LocalAccount.shortcut_fn.Shortcut_NoAccount': ( 'LocalAccount/shortcut_fn.html#shortcut_noaccount',
                                                                                                                                       'domolibrary_execution/LocalAccount/shortcut_fn.py'),
                                                                'domolibrary_execution.LocalAccount.shortcut_fn.Shortcut_NoAccount.__init__': ( 'LocalAccount/shortcut_fn.html#shortcut_noaccount.__init__',
                                                                                                                                                'domolibrary_execution/LocalAccount/shortcut_fn.py'),
                                                                'domolibrary_execution.LocalAccount.shortcut_fn.share_domo_account_with_domo_group': ( 'LocalAccount/shortcut_fn.html#share_domo_account_with_domo_group',
                                                                                                                                                       'domolibrary_execution/LocalAccount/shortcut_fn.py')},
            'domolibrary_execution.client': { 'domolibrary_execution.client.ExecutionError': ( 'client.html#executionerror',
                                                                                               'domolibrary_execution/client.py'),
                                              'domolibrary_execution.client.ExecutionError.__init__': ( 'client.html#executionerror.__init__',
                                                                                                        'domolibrary_execution/client.py')},
            'domolibrary_execution.client.ExecutionError': { 'domolibrary_execution.client.ExecutionError.ExecutionError': ( 'client/executionerror.html#executionerror',
                                                                                                                             'domolibrary_execution/client/ExecutionError.py'),
                                                             'domolibrary_execution.client.ExecutionError.ExecutionError.__init__': ( 'client/executionerror.html#executionerror.__init__',
                                                                                                                                      'domolibrary_execution/client/ExecutionError.py')},
            'domolibrary_execution.process': { 'domolibrary_execution.process.ProcessFunction_Error': ( 'process.html#processfunction_error',
                                                                                                        'domolibrary_execution/process.py'),
                                               'domolibrary_execution.process.ProcessFunction_Error.__init__': ( 'process.html#processfunction_error.__init__',
                                                                                                                 'domolibrary_execution/process.py'),
                                               'domolibrary_execution.process.ProcessFunction_ResponseTypeError': ( 'process.html#processfunction_responsetypeerror',
                                                                                                                    'domolibrary_execution/process.py'),
                                               'domolibrary_execution.process.ProcessFunction_ResponseTypeError.__init__': ( 'process.html#processfunction_responsetypeerror.__init__',
                                                                                                                             'domolibrary_execution/process.py'),
                                               'domolibrary_execution.process.ResponseProcess': ( 'process.html#responseprocess',
                                                                                                  'domolibrary_execution/process.py'),
                                               'domolibrary_execution.process.ResponseProcess.to_json': ( 'process.html#responseprocess.to_json',
                                                                                                          'domolibrary_execution/process.py'),
                                               'domolibrary_execution.process.process_function': ( 'process.html#process_function',
                                                                                                   'domolibrary_execution/process.py')},
            'domolibrary_execution.utils.domojupyter': { 'domolibrary_execution.utils.domojupyter.DJ_Workspace_InvalidCreds': ( 'utils/domojupyter.html#dj_workspace_invalidcreds',
                                                                                                                                'domolibrary_execution/utils/domojupyter.py'),
                                                         'domolibrary_execution.utils.domojupyter.DJ_Workspace_InvalidCreds.__init__': ( 'utils/domojupyter.html#dj_workspace_invalidcreds.__init__',
                                                                                                                                         'domolibrary_execution/utils/domojupyter.py'),
                                                         'domolibrary_execution.utils.domojupyter.DJ_Workspace_NoAuth': ( 'utils/domojupyter.html#dj_workspace_noauth',
                                                                                                                          'domolibrary_execution/utils/domojupyter.py'),
                                                         'domolibrary_execution.utils.domojupyter.DJ_Workspace_NoAuth.__init__': ( 'utils/domojupyter.html#dj_workspace_noauth.__init__',
                                                                                                                                   'domolibrary_execution/utils/domojupyter.py'),
                                                         'domolibrary_execution.utils.domojupyter.generate_domo_auth': ( 'utils/domojupyter.html#generate_domo_auth',
                                                                                                                         'domolibrary_execution/utils/domojupyter.py'),
                                                         'domolibrary_execution.utils.domojupyter.generate_full_auth': ( 'utils/domojupyter.html#generate_full_auth',
                                                                                                                         'domolibrary_execution/utils/domojupyter.py'),
                                                         'domolibrary_execution.utils.domojupyter.generate_token_auth': ( 'utils/domojupyter.html#generate_token_auth',
                                                                                                                          'domolibrary_execution/utils/domojupyter.py'),
                                                         'domolibrary_execution.utils.domojupyter.get_creds_from_domojupyter_account': ( 'utils/domojupyter.html#get_creds_from_domojupyter_account',
                                                                                                                                         'domolibrary_execution/utils/domojupyter.py'),
                                                         'domolibrary_execution.utils.domojupyter.read_domo_jupyter_account': ( 'utils/domojupyter.html#read_domo_jupyter_account',
                                                                                                                                'domolibrary_execution/utils/domojupyter.py'),
                                                         'domolibrary_execution.utils.domojupyter.remove_asterix': ( 'utils/domojupyter.html#remove_asterix',
                                                                                                                     'domolibrary_execution/utils/domojupyter.py'),
                                                         'domolibrary_execution.utils.domojupyter.which_environment': ( 'utils/domojupyter.html#which_environment',
                                                                                                                        'domolibrary_execution/utils/domojupyter.py')},
            'domolibrary_execution.utils.factory': { 'domolibrary_execution.utils.factory.FactoryConfig': ( 'factory/factory.html#factoryconfig',
                                                                                                            'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryConfig.asdict': ( 'factory/factory.html#factoryconfig.asdict',
                                                                                                                   'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryConfig.run': ( 'factory/factory.html#factoryconfig.run',
                                                                                                                'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryConfig.to_json': ( 'factory/factory.html#factoryconfig.to_json',
                                                                                                                    'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryFunction_Error': ( 'factory/factory.html#factoryfunction_error',
                                                                                                                    'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryFunction_Error.__init__': ( 'factory/factory.html#factoryfunction_error.__init__',
                                                                                                                             'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryFunction_MissingParameter': ( 'factory/factory.html#factoryfunction_missingparameter',
                                                                                                                               'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryFunction_MissingParameter.__init__': ( 'factory/factory.html#factoryfunction_missingparameter.__init__',
                                                                                                                                        'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryFunction_NotSuccess': ( 'factory/factory.html#factoryfunction_notsuccess',
                                                                                                                         'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryFunction_NotSuccess.__init__': ( 'factory/factory.html#factoryfunction_notsuccess.__init__',
                                                                                                                                  'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryFunction_ResponseError': ( 'factory/factory.html#factoryfunction_responseerror',
                                                                                                                            'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryFunction_ResponseError.__init__': ( 'factory/factory.html#factoryfunction_responseerror.__init__',
                                                                                                                                     'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryFunction_ResponseTypeError': ( 'factory/factory.html#factoryfunction_responsetypeerror',
                                                                                                                                'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryFunction_ResponseTypeError.__init__': ( 'factory/factory.html#factoryfunction_responsetypeerror.__init__',
                                                                                                                                         'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryLogs': ( 'factory/factory.html#factorylogs',
                                                                                                          'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryLogs.add_response': ( 'factory/factory.html#factorylogs.add_response',
                                                                                                                       'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryLogs.to_json': ( 'factory/factory.html#factorylogs.to_json',
                                                                                                                  'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryMessage': ( 'factory/factory.html#factorymessage',
                                                                                                             'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryMessage.__eq__': ( 'factory/factory.html#factorymessage.__eq__',
                                                                                                                    'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryMessage.to_json': ( 'factory/factory.html#factorymessage.to_json',
                                                                                                                     'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryMessage.to_string': ( 'factory/factory.html#factorymessage.to_string',
                                                                                                                       'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryResponse': ( 'factory/factory.html#factoryresponse',
                                                                                                              'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryResponse.__eq__': ( 'factory/factory.html#factoryresponse.__eq__',
                                                                                                                     'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryResponse.add_message': ( 'factory/factory.html#factoryresponse.add_message',
                                                                                                                          'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryResponse.test_success': ( 'factory/factory.html#factoryresponse.test_success',
                                                                                                                           'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.FactoryResponse.to_json': ( 'factory/factory.html#factoryresponse.to_json',
                                                                                                                      'domolibrary_execution/utils/factory.py'),
                                                     'domolibrary_execution.utils.factory.factory_function': ( 'factory/factory.html#factory_function',
                                                                                                               'domolibrary_execution/utils/factory.py')}}}



"""Fill in a module description here"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/domojupyter.ipynb.

# %% auto 0
__all__ = ['which_environment', 'DJ_Workspace_NoAuth', 'read_domo_jupyter_account', 'get_creds_from_domojupyter_account',
           'DJ_Workspace_InvalidCreds', 'generate_token_auth', 'generate_full_auth', 'generate_domo_auth']

# %% ../../nbs/utils/domojupyter.ipynb 2
from typing import Callable

import os
import json

from operator import itemgetter
import urllib.parse as urllib_parse

import domolibrary_execution.client.ExecutionError as dxde

import domolibrary.client.DomoError as dmde
import domolibrary.client.DomoAuth as dmda
import domolibrary.classes.DomoAccount as dmacc
import domolibrary.classes.DomoInstanceConfig as dmic
import domolibrary.classes.DomoUser as dmdu
import domolibrary.classes.DomoRole as dmdr

import domolibrary.utils.chunk_execution as dmce
import domolibrary_extensions.utils.execution as dtex


from nbdev.showdoc import patch_to

# %% ../../nbs/utils/domojupyter.ipynb 4
def which_environment():
    return urllib_parse.urlparse(os.environ.get("DOMO_HOSTNAME")).netloc.replace(
        ".domo.com", ""
    )

# %% ../../nbs/utils/domojupyter.ipynb 5
class DJ_Workspace_NoAuth(dxde.ExecutionError):
    def __init__(self, account_name, message):
        super().__init__(f"{account_name} - {message}")


@dtex.auto_retry_sync()
def read_domo_jupyter_account(
    account_name,
    domojupyter_fn: Callable,
    is_abstract: bool = False,
    is_dict: bool = True,
):
    creds = None
    try:
        account_properties = domojupyter_fn.get_account_property_keys(account_name)

        creds = {
            prop: domojupyter_fn.get_account_property_value(account_name, prop)
            for prop in account_properties
        }
    except Exception as e:
        raise DJ_Workspace_NoAuth(
            account_name=account_name,
            message= f"🤯🤯 unable to retrieve {account_name} - {e}",
        ) from e

    if not is_abstract:
        return creds

    creds = creds["credentials"]

    if not is_dict:
        return creds.strip()

    return json.loads(creds)

# %% ../../nbs/utils/domojupyter.ipynb 6
def remove_asterix(text):
    if text == "" or text == "********":
        return None
    return text

# %% ../../nbs/utils/domojupyter.ipynb 7
def get_creds_from_domojupyter_account(
    account_name: str,
    domojupyter_fn: Callable,
    is_abstract: bool,
) -> dict:

    creds = read_domo_jupyter_account(
        account_name, domojupyter_fn, is_abstract=is_abstract
    )

    creds = {
        "access_token": (
            creds.get("DOMO_ACCESS_TOKEN")
            if is_abstract
            else creds.get("domoAccessToken")
        ),
        "password": (
            creds.get("DOMO_PASSWORD") if is_abstract else creds.get("password")
        ),
        "username": (
            creds.get("DOMO_USERNAME") if is_abstract else creds.get("username")
        ),
    }

    creds = {key: remove_asterix(value) for key, value in creds.items()}

    return creds

# %% ../../nbs/utils/domojupyter.ipynb 8
class DJ_Workspace_InvalidCreds(dxde.ExecutionError):
    def __init__(self, account_name, message):
        super().__init__(f"{account_name} - {message}")


async def generate_token_auth(
    domo_instance: str,
    domojupyter_fn: Callable,
    account_name: str = None,
    is_include_domo_account: bool = False,
    is_abstract: bool = False,
):

    account_name = account_name or f"sdk_{domo_instance}"

    creds = get_creds_from_domojupyter_account(
        account_name=account_name,
        domojupyter_fn=domojupyter_fn,
        is_abstract=is_abstract,
    )

    domo_access_token = creds.get("access_token")

    if not domo_access_token:
        raise DJ_Workspace_InvalidCreds(
            account_name=account_name, message="no access token stored in account"
        )

    auth = dmda.DomoTokenAuth(
        domo_instance=domo_instance, domo_access_token=domo_access_token
    )

    await auth.print_is_token()

    if is_include_domo_account:
        domo_accounts = await dmacc.DomoAccounts.get_accounts(auth=auth)
        domo_account = next((acc for acc in domo_accounts if acc.name == account_name))
        return auth, domo_account

    return auth

# %% ../../nbs/utils/domojupyter.ipynb 9
async def generate_full_auth(
    domo_instance: str,
    domojupyter_fn: Callable,
    account_name : str = None, #defauls to sdk_domo_instance
    is_include_domo_account: bool = False,
    is_abstract: bool = False,
):

    account_name = account_name or f"sdk_{domo_instance}"

    creds = get_creds_from_domojupyter_account(
        account_name=account_name,
        domojupyter_fn=domojupyter_fn,
        is_abstract=is_abstract,
    )

    domo_username = creds.get("username")
    if not domo_username:
        raise DJ_Workspace_InvalidCreds(
            account_name=account_name, message="no username stored in account"
        )

    domo_password = creds.get("password")
    if not domo_password:
        raise DJ_Workspace_InvalidCreds(
            account_name=account_name, message="no password stored in account"
        )

    auth = dmda.DomoFullAuth(
        domo_instance=domo_instance,
        domo_username=domo_username,
        domo_password=domo_password,
    )

    await auth.print_is_token()

    if is_include_domo_account:
        domo_accounts = await dmacc.DomoAccounts.get_accounts(auth=auth)
        domo_account = next((acc for acc in domo_accounts if acc.name == account_name))
        return auth, domo_account

    return auth

# %% ../../nbs/utils/domojupyter.ipynb 10
async def generate_domo_auth(
    domo_instance: str,
    domojupyter_fn: Callable,
    account_name : str = None,
    is_include_domo_account: bool = False,
    is_abstract: bool = False,
    backup_auth: dmda.DomoAuth = None
):
    account_name = account_name or f"sdk_{domo_instance}"

    try:
        return await generate_full_auth(
            account_name= account_name,
            domo_instance=domo_instance,
            domojupyter_fn=domojupyter_fn,
            is_include_domo_account=is_include_domo_account,
            is_abstract=is_abstract,
        )
    except (dmde.DomoError, dxde.ExecutionError) as e:
        print(e)
        pass

    try:        
        return await generate_token_auth(
            account_name= account_name,
            domo_instance=domo_instance,
            domojupyter_fn=domojupyter_fn,
            is_include_domo_account=is_include_domo_account,
            is_abstract=is_abstract,
        )
    except (dmde.DomoError, dxde.ExecutionError) as e:
        print(e)
        pass

    if not backup_auth:
        raise DJ_Workspace_NoAuth(account_name = account_name, message = f"no valid creds in {account_name}.  Pass backup account or update creds")

    if not backup_auth.is_valid_token:
        await backup_auth.print_is_token()

    return backup_auth




    



"""Fill in a module description here"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/LocalAccount/shortcut_fn.ipynb.

# %% auto 0
__all__ = ['Shortcut_NoAccount', 'share_domo_account_with_domo_group']

# %% ../../nbs/LocalAccount/shortcut_fn.ipynb 2
from typing import List
import asyncio

import domolibrary_execution.utils.domojupyter as dxdj
import domolibrary_execution.client.ExecutionError.py as dxee

import domolibrary.client.DomoAuth as dmda
import domolibrary.classes.DomoGroup as dmdg
import domolibrary.classes.DomoAccount as dmacc
# import domolibrary.client.DomoError as dmde


# import domolibrary.utils.chunk_execution as dmce



# %% ../../nbs/LocalAccount/shortcut_fn.ipynb 8
class Shortcut_NoAccount(dxee.ExecutionError):
    def __init__(self, account_name, domo_instance):
        super().__init__(
            f"unable to retrieve account - {account_name} from {domo_instance}"
        )


async def share_domo_account_with_domo_group(
    auth: dmda.DomoAuth, 
    account_name : str,
    group_name: str,
    is_search_system_accounts : bool = True
                                 ):
    
    domo_group = await dmdg.DomoGroups.upsert(auth = auth,
                                              group_name = group_name,
                                             is_search_system_accounts = is_search_system_accounts)
    
    domo_account = await search_domo_account_by_name(auth = auth, account_name = account_name)

    return await domo_account.share(group_id=share_domo_group.id, auth=auth)




"""Fill in a module description here"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/client/ExecutionError.ipynb.

# %% auto 0
__all__ = ['ExecutionError']

# %% ../../nbs/client/ExecutionError.ipynb 3
class ExecutionError(Exception):
    def __init__(self, message):
        super().__init__(message)
